C LP.for:
C
C Purpose: Sovle the limited participation model of Christiano, Eichenbaum
C          and Evans (1997) using the parameterized expectations approach (PEA)
C          The solution is computed from the solution to
C          a system of non-linear equations.
C          Genetic search is used to find good starting values.


C Module definitions
      Module Def1
	
	Real(8) a, alpha, beta, eta, delta, nue, nstar, theta	
	Real(8) rhoZ, sigmaZ, rhoMu, sigmaMu, mustar
	Real(8) kstar, xstar, cstar, qstar
	Integer(4) nobs, ta, ng, nrx, ncx, nu, nl
	Real(8) MaxExp1, MaxExp2, MinExp1
	Real(8), allocatable :: x(:,:), u(:,:), eps(:,:), lq(:,:)
      Integer(4) Search, PsiError(2)

	End Module Def1

C	Matrices are generated with LP2.For and stored in
C	binary files lp_lxx.bin, lp_lxz.bin,...
C	files are copied from \LP2 into \LP
C

	Module Mat1

	Real(8) Llx(2,3), Llz(2,2),
     +        Lxx(3,3), Lxz(3,2), Lux(4,3), Luz(4,2)

	End Module Mat1

C Here begins the main program
      program LP

      use DFLIB
	use IMSLF90
	use Mat1
	
      use Def1
	use MNR
	

      Integer(4) t, rc, lags, nums, maxs, nofs, freq(3), length
	Integer(2) row
	Real(4) Asec
      Real(8), Allocatable :: Gamma1(:), Gamma2(:)
	Real(8) yk, ystar, lstar, gstar
	Real(8) StDev, dm, chi(2), fit, SSE(3), crit(1:4)	

	Character(256) Laufzeit
	Character(50) MyDate

	Logical initialize, ok, InitialSettings

      External Fita, Fitb
      External Psi, dPsi

	Type (rccoord) curpos		

C Read Parameters from input file and compute derived parameters
C Parameters of the algorithm
      Call GetL('initialize','Parameters.txt',initialize)
	Call GetI('nobs' ,'Parameters.txt',nobs )
	Call GetI('ta'   ,'Parameters.txt',ta   )	
	Call GetI('search','Parameters.txt',search)
	
      Call GetD('a'     ,'Parameters.txt',a)
	Call GetD('alpha' ,'Parameters.txt',alpha)
	Call GetD('beta'  ,'Parameters.txt',beta )
	Call GetD('eta'  ,'Parameters.txt',eta ) 
	Call GetD('delta','Parameters.txt',delta)	
	Call GetD('nue'   ,'Parameters.txt',nue)
	Call GetD('nstar' ,'Parameters.txt',nstar)
	Call GetD('rhoz'  ,'Parameters.txt',rhoZ  )
	Call GetD('sigmaz','Parameters.txt',sigmaZ)
      Call GetD('rhomu'  ,'Parameters.txt',rhoMu  )
	Call GetD('sigmamu','Parameters.txt',sigmaMu)
	Call GetD('mustar' ,'Parameters.txt',mustar)

C Assign hard wired parameter values      
      ng=18
	ncx=5
	nu=4
	nl=2
	
	nrx=nobs+ta+1
	MaxExp1=2.0
	MaxExp2=10.0
      MaxExp1=dlog(MaxExp1)
	MaxExp2=dlog(MaxExp2)
	MinExp1=MaxExp1/MaxExp2
	t=MaxExponent(MinExp1)

	MaxExp2=0.95*dble(t*MinExp1)
	MaxExp2=0.5*MaxExp2
	MaxExp1=0.95*dble(t*MaxExp1)
	t=MinExponent(MinExp1)
	MinExp1=2.0
	MinExp1=dlog(MinExp1)
	MinExp1=0.95*dble(t*MinExp1)
	
      
C Allocate space to u, x, lq, gamma, and eps
      Allocate(u(1:nrx-1,1:nu), lq(1:nrx-1,1:nl), x(1:nrx,1:ncx))	 
      Allocate(Gamma1(1:ng), Gamma2(1:ng))
	Allocate(eps(1:nrx,1:2))
	
C Prepare output window
      ok=InitialSettings()
      Call MakeSmallWindow(10,"Program Progress",2)

      
C Fix starting time
      Asec=Secnds(0.0)
      
	
	open(25,form='unformatted',file='lp_lxx.bin')
	read(25) Lxx
	close(25)

	open(26,form='unformatted',file='lp_lxz.bin')
	read(26) Lxz
	close(26)

	open(27,form='unformatted',file='lp_lux.bin')
	read(27) Lux
	close(27)

	open(28,form='unformatted',file='lp_luz.bin')
	read(28) Luz
	close(28)

	open(29,form='unformatted',file='lp_llx.bin')
	read(29) Llx
	close(29)

	open(30,form='unformatted',file='lp_llz.bin')
	read(30) Llz
	close(30)


      open(11,file='LogFile.txt')	
	open(15,file='Summary.txt')

	WRITE(11,"(A50)") MyDate()	
	WRITE(15,"(A50)") MyDate()

C Initiatize shocks
      Inquire(File='eps.bin',Exist=ok)
      If ((.not. initialize) .and. (ok) ) then
	     open(20,form='unformatted',file='eps.bin')
	     read(20), eps
	     close(20)
      else
  	   Call SetTextPosition(1,1,curpos)
	   WRITE(10,"('Initialize Shocks')")	   
	   Call DRNNOR(nrx,eps(1:nrx,1)) ! Draw shocks from a standard normal
	   Call DRNNOR(nrx,eps(1:nrx,2))
         open(20, form='unformatted',file='eps.bin')
	   write(20) eps
	   close(20)
	   Call SetTextPosition(1,1,curpos)
	   Write(10,"('                   ')")	   
      endif
		
	

C Compute sequence of productivity shocks
      X(1,4)=exp(SigmaZ*eps(1,1))
	X(1,5)=mustar*exp(SigmaMu*eps(1,2))
      Do t=1,nrx-1
	   X(t+1,4)=(X(t,4)**Rhoz)*exp(Sigmaz*eps(t+1,1))
	   X(t+1,5)=(mustar**(1.-RhoMu))*(X(t,5)**Rhomu)
     +   	        *exp(Sigmamu*eps(t+1,2))	   
	End Do


C Compute stationary solution and derived parameters
      yk=((a**(eta)) - beta*(1.-delta))/(beta*(1.-alpha))
	kstar=nstar*(yk**(-1./alpha))
	ystar=kstar*yk
	cstar=ystar-(a+delta-1.)*kstar	
	qstar=(mustar/beta)*(a**(eta-1.))
	xstar=mustar*(cstar-((alpha*ystar)/qstar))
      theta=(alpha/nue)*(ystar/qstar)*(nstar**(-nue))

C Test solution by solving for c,N,pi, and q given (k,m,x,z,mu,lambda,gamma)

C     lstar=((cstar-theta*(nstar**nue))**(-eta))*(1./qstar)
C	gstar=lstar*(qstar-1.)
C     x(1,1)=kstar
C	x(1,2)=cstar
C	x(1,3)=xstar
C	x(1,4)=1.0
C	x(1,5)=mustar
C	lq(1,1)=lstar
C	lq(1,2)=qstar

C     Call GetU(1,rc)

      if (search .eq. 0) then	      			      	  
C	initialization of Gamma with the help of the log-linear
C	solution from LP2.For
C

C	First Euler equation: lambda

	  Call FixvMN(ng,Gamma1,FitA,Gamma2,rc,crit)	      	  	  
        goto 7
	endif


C  find good starting values
      maxs=10      

C Try to find solution
      Do nums=1,maxs           
         if (search .eq. 1) Call GSearch1(Fitb,ng,Gamma1,Fit)
         if (search .eq. 2) Call GSearch2(Fitb,ng,Gamma1,Fit)
   	   Call ClearScreen($GCLEARSCREEN)        
	   CALL SETTEXTPOSITION(1,2,curpos)
	   WRITE(10,"('Simulation No. ',I3)") nums
	   WRITE(11,"('Simulation No. ',I3)") nums	
	   CALL SETTEXTPOSITION(2,2,curpos)
	   WRITE(10,"('Solution from GSearch with Fit=',1X,F15.8)") Fit
	   WRITE(11,"('Solution from GSearch with Fit=',1X,F15.8)") Fit	
	   CALL SETTEXTPOSITION(3,2,curpos)
	   WRITE(10,*) 'Found Vector:'
	   WRITE(11,*) 'Found Vector:'
	   row=4		
	   DO t=1,ng
	     CALL SETTEXTPOSITION(row,2,curpos)	   
	     WRITE(10,"(F12.8)") Gamma1(t)
	     WRITE(11,"(F12.8)") Gamma1(t)	     
	     row=row+1 
	    END DO
C         if (fit .lt. 1.0E-7) goto 8
	   Call Fita(ng,ng,Gamma1,Fit)
	   Call Fitb(ng,Gamma1,Fit)
         Call FixvMN(ng,Gamma1,Fita,Gamma2,rc,crit) !refine solution
	    
	  Call SetTextPosition(row+1,2,curpos)
	  if (rc .eq. 1) then
	   WRITE(10,"('Error Message from FixvMN: ',A,' Psi:',2I)")
     +   MNR_Message(rc), PsiError(1:2)
	  WRITE(11,"('Error Message from FixvMN: ',A,' Psi:',2I)")
     + MNR_Message(rc), PsiError(1:2)
      else
         WRITE(10,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	   WRITE(11,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	endif
	  Do t=1,ng
         Call SetTextPosition(row,2,curpos)	    
	   Write(11,"(F10.4)") Gamma2(t)
	   row=row+1
	  End do             
	  
        if (rc .eq. 0) exit
	 
	End Do
	
      
7     Call ElapsedTime(Secnds(Asec),Laufzeit)      
	Write(15,"('Run time=',A)") Laufzeit
	if (nums .gt. maxs) then
	 Call SetTextPosition(row+1,2,curpos)
	 WRITE(10,"('Not able to find solution after',I3,' Iterations')")
     +  maxs
      else
	  Write(15,"('Error Message from FixVMN',A)") MNR_Message(rc)
	  Write(15,"('abs(f(i))=',E15.7)") crit(1)
	  Write(15,"('abs(dx/x)=',E15.7)") crit(2)
	  Write(15,"('GradTest =',E15.7)") crit(3)
	  Write(15,"('0.5*f*f  =',E15.7)") crit(4)      
	  Write(15,"('Final Solution')")
        Do t=1,ng         
	   Write(15,"(F10.4)") Gamma2(t)
	   row=row+1
	  End do             
	endif
         Write(15,"('Parameters of the model:')")
	   Write(15,"('alpha  =  ',F7.3)") alpha
	   Write(15,"('beta   =  ',F7.3)") beta
	   Write(15,"('eta    =  ',F7.3)") eta
	   Write(15,"('delta  =  ',F7.3)") delta
	   Write(15,"('nue     =  ',F7.3)") nue
	   Write(15,"('nstar  =  ',F7.3)") nstar
	   Write(15,"('rhoZ   =  ',F7.3)") rhoZ
	   Write(15,"('sigmaZ =  ',F7.4)") sigmaZ
	   Write(15,"('rhoMu  =  ',F7.3)") rhoMu
	   Write(15,"('sigmaMu=  ',F7.4)") sigmaMu
	   Write(15,"('Parameters of the simulations:')")
	   
	   Write(15,"('nobs=  ',I6)") nobs
         Write(15,"('ta  =  ',I6)") ta
	
C Various tests
8     if (rc .eq. 0) then         
	   Call GetSSE(Gamma2,SSE)
	   Write(15,"('Sum of squarred erros:')")
	   Do t=1,3
	      Write(15,"('SSE(',I1,')=',F20.15)") t, SSE(t)
	   end do

        Call MyStat(nrx,x(1:nrx,4),crit)
  	  write(15,"('zbar=',F12.8,' stdz=',F12.8)") crit(3), crit(4)
	  Call MyStat(nrx,x(1:nrx,5),crit)
	  write(15,"('mustar=',F12.8,' mubar=',F12.8,' stdmu=',F12.8)")
     +	     mustar,crit(3), crit(4)
        Call MyStat(nrx,x(1:nrx,1),crit)
        write(15,"('kstar=',F12.8,' kbar=',F12.8,' stdk=',F12.8)")
     +  x(1,1),crit(3), crit(4)
	  Call MyStat(nrx,x(1:nrx,2),crit)
        write(15,"('mstar=',F12.8,' mbar=',F12.8,' stdm=',F12.8)")
     +  x(1,2), crit(3), crit(4)
	  Call MyStat(nrx-1,u(1:nrx-1,1),crit)
	  write(15,"('cstar=',F12.8,' cbar=',F12.8,' stdc=',F12.8)")
     +	  cstar,crit(3), crit(4)
        Call MyStat(nrx,x(1:nrx,3),crit)
	  write(15,"('xstar=',F12.8,' xbar=',F12.8,' stdx=',F12.8)")
     +	  x(1,3),crit(3), crit(4)

	  Write(15,"('Std consumption=',F10.5)") StDev(nrx-1,u(1:nrx-1,1))
	  Write(15,"('Std hours      =',F10.5)") StDev(nrx-1,u(1:nrx-1,2))
	  Write(15,"('Std. Inflation =',F10.5)") StDev(nrx-1,u(1:nrx-1,3))
        Write(15,"('Number of binding constraints:',I5)")
     +    Count(u(1:nrx-1,4) .gt. 0.0)

        Call Test(Gamma2)
	endif
      close(11)
	close(15)
5	End


C **************************  Subroutines ********************************
C
C
C
C Psi1 returns Psi(Gamma,x(t)) in the case of a first degree polynomial
      Real(8) Function Psi(gamma,xt)

	use Def1
	use MNR

      Real(8) gamma(6), xt(ncx)
      

      Psi=Gamma(1)+Gamma(2)*dlog(xt(1))+Gamma(3)*dlog(xt(2))+
     +    Gamma(4)*dlog(xt(3))+Gamma(5)*dlog(xt(4))+Gamma(6)*dlog(xt(5))

	
      PsiError=0
      If (Psi .gt. MaxExp1) then
	   FError=1
	   PsiError(1)=1
	   return
	else
	   Psi=beta*(a**(-eta))*exp(Psi)
	   if (Psi .le. Epsilon(a)) FError=1
	endif      

	Return
	End Function

C dPsi: computes the gradient of Psi1 in the case of a first degree polynomial
      Subroutine dPsi(Gamma,xt,g)

      use Def1
      use MNR

	Real(8) Gamma(6), xt(1:ncx), g(6), Psi

      External Psi

      g(1)=Psi(Gamma,xt)
	g(2)=g(1)*dlog(xt(1))
	g(3)=g(1)*dlog(xt(2))
	g(4)=g(1)*dlog(xt(3))	
	g(5)=g(1)*dlog(xt(4))
	g(6)=g(1)*dlog(xt(5))	

      Return

	End Subroutine

C Getu: Solves for u:=(c,N,pi,gamma) given x:=(k,m,x,z,mu) and lq:=(lambda,q)

      Subroutine GetU(t,rc)

      use Def1	

	Integer(4) t, rc
	
	Real(8) n, y, c, p

      rc=0

C First: Compute n and p (the inflation factor), which are independent of g
      n=(alpha/(theta*nue))*x(t,4)*(x(t,1)**(1.-alpha))*(1./lq(t,2))
	n=n**(1./(nue-alpha))
      y=x(t,4)*(n**alpha)*(x(t,1)**(1.-alpha))
	p=((x(t,5)*x(t,2)-x(t,3))/(a*alpha*y))*lq(t,2)
	
	if (p .lt. 0.0) then
	   rc=1
	   return
	endif
	

C compute c for gamma=0
      c=(lq(t,1)**(-1./eta))+theta*(n**nue)

      if ((c-theta*(n**(nue))) .lt. 0.0) then
	    rc=1
	    return
	endif

      
C Check whether the cash in advance constraint binds
	if (c .lt. (x(t,5)*x(t,2))/(a*p) ) then
	   u(t,1)=c
	   u(t,2)=n
	   u(t,3)=p
	   u(t,4)=0.0
	else	   
	   c=(x(t,5)*x(t,2))/(a*p)
         u(t,1)=c
	   u(t,2)=n
	   u(t,3)=p	   
	   u(t,4)=((c-theta*(n**nue))**(-eta))-lq(t,1)
	endif

	return

	End Subroutine


C Fit1: Computes the rhs of the system of equations, whose zero is the
C       PEA solution in the case of first degree polynomial

      Subroutine FitA(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j, rc, ng1
	Real(8) temp(3), Psi

	External Psi
      
	ng1=6 !number of elements of PSI(.)

	FError=0
	

      x(1,1)=kstar
	x(1,2)=cstar
	x(1,3)=xstar

	do t=1,nrx-1
             lq(t,1)=Psi(gamma(1:ng1),x(t,1:ncx))                 !lambda(t)
	       lq(t,2)=Psi(gamma(ng1+1:2*ng1),x(t,1:ncx))/lq(t,1)    !q(t)
	      if (lq(t,2) .lt. 1.0) then
	          FError=1
	          return
	      endif
	      x(t+1,3)=Psi(gamma(2*ng1+1:3*ng1),x(t,1:ncx))/lq(t,1) !x(t+1)
	      if (x(t+1,3) .lt. Epsilon(a)) then
	       FError=1
	       return
	      endif            	      
	       
	      Call GetU(t,rc) !solve for c,N,pi,and gamma, results are stored in u(t,1:4)
	      if (rc .ne. 0) then
	        FError=1
	        return
	      endif
	    
	     x(t+1,1)=(1./a)* (x(t,4)*(u(t,2)**alpha)*(x(t,1)**(1.-alpha))
     +                    +(1.-delta)*x(t,1) - u(t,1) )  !k(t+1)
	     if (x(t+1,1) .le. Epsilon(a)) then
	       FError=1
	       return
	     endif
	     x(t+1,2)=(x(t,5)*x(t,2))/(a*u(t,3)) !m(t+1)
	     if (x(t+1,2) .lt. Epsilon(a)) then
	       FError=1
	       return
 	     endif
	
	end do

      fit=0.0

	do t=ta,nrx-2

        temp(1)=beta*(a**(-eta))*lq(t+1,1)*
     +         (1.-delta+(1.-alpha)*x(t+1,4)*(u(t+1,2)**alpha)
     +          *(x(t+1,1)**(-alpha))) - PSI(Gamma(1:ng1),x(t,1:ncx))
        temp(2)=beta*(a**(-eta))*lq(t+1,1)*lq(t+1,2)*lq(t,2)	  
     +          *(1./u(t+1,3)) - PSI(Gamma(ng1+1:2*ng1),x(t,1:ncx))
	  temp(3)=beta*(a**(-eta))*(x(t+1,3))*(lq(t+1,1)+u(t+1,4))
     +          *(1./u(t+1,3)) - PSI(Gamma(2*ng1+1:3*ng1),x(t,1:ncx))
	  
	   Call dPsi(Gamma(1:ng1),x(t,1:ncx),g(1:ng1))
	   Call dPsi(Gamma(ng1+1:2*ng1),x(t,1:ncx),g(ng1+1:2*ng1))
	   Call dPsi(Gamma(2*ng1+1:3*ng1),x(t,1:ncx),g(2*ng1+1:3*ng1))

         do j=1,ng1
	      fit(j)      =fit(j)       +temp(1)*g(j)
	      fit(j+ng1)  =fit(j+ng1)   +temp(2)*g(j+ng1)
	      fit(j+2*ng1)=fit(j+2*ng1) +temp(3)*g(j+2*ng1)
	   end do
	   	      
      End Do

      if (MaxVal(dabs(fit)) .gt. (10.**MaxExp2)) FError=1
	
	Return

	End Subroutine


C Fitb(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fita

      Subroutine FitB(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call FitA(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
      Return
	END SUBROUTINE


C Test: Computes the steady state values of the model from the parameterized equation,
C       computes the percentage deviation of these solutions to the exact ones, and
C       write the result to the file summary.txt

      Subroutine Test(Gamma)

	use Def1
	Real(8) Gamma(ng)

      Integer(4) rc, t, ng1
      Real(8) Psi, dx(7), pistar, temp, lstar, gstar
      

	External Psi

      ng1=6
	t=1	
	pistar=mustar/a
	lstar=((cstar-theta*(nstar**nue))**(-eta))*(1./qstar)
      gstar=lstar*(qstar-1.)
	        x(1,1)=kstar
              x(1,2)=cstar !mstar
              x(1,3)=xstar
	        x(1,4)=1.0
	        x(1,5)=mustar
              
             lq(1,1)=Psi(gamma(1:ng1),x(1,1:ncx))                !lambda
	       lq(1,2)=Psi(gamma(ng1+1:2*ng1),x(1,1:ncx))/lq(1,1)  !q
	       temp   =Psi(gamma(2*ng1+1:3*ng1),x(1,1:ncx))/lq(1,1)!x
		   
		   Call GetU(t,rc) !solve for c, N, pi, and gamma    	      
	       dx(1)=(u(1,1)-cstar)/cstar
	       dx(2)=(u(1,2)-nstar)/nstar
	       dx(3)=(u(1,3)-pistar)/pistar
	       dx(4)=(lq(1,1)-lstar)/lstar
	       dx(5)=(lq(1,2)-qstar)/qstar
	       dx(6)=(u(1,4)-gstar)/gstar	       
	       dx(7)=(temp-xstar)/xstar
		   write(15,"('(dc/c)= ',F12.6)") dx(1)
		   write(15,"('(dN/N)= ',F12.6)") dx(2) 
	       write(15,"('(dp/p)= ',F12.6)") dx(3)
	       write(15,"('(dl/l)= ',F12.6)") dx(4)
		   write(15,"('(dq/q)= ',F12.6)") dx(5)
		   write(15,"('(dg/g)= ',F12.6)") dx(6)
	       write(15,"('(dx/x)= ',F12.6)") dx(7)
      Return

	End Subroutine

C GetSSE: Computes sum of squarred errors
      Subroutine GetSSE(Gamma,SSE)

	use Def1
      use MNR

	Real(8) Gamma(ng), SSE(3)

      Real(8) temp(3), Psi
      Integer(4) ng1, t, rc

	External Psi

      x(1,1)=kstar
	x(1,2)=cstar
	x(1,3)=xstar
	ng1=6
	FError=0

	do t=1,nrx-1
             lq(t,1)=Psi(gamma(1:ng1),x(t,1:ncx))                 !lambda(t)
	       lq(t,2)=Psi(gamma(ng1+1:2*ng1),x(t,1:ncx))/lq(t,1)    !q(t)
	      if (lq(t,2) .lt. 1.0) then
	          FError=1
	          return
	      endif
	      x(t+1,3)=Psi(gamma(2*ng1+1:3*ng1),x(t,1:ncx))/lq(t,1) !x(t+1)
	      if (x(t+1,3) .lt. Epsilon(a)) then
	       FError=1
	       return
	      endif            	      
	       
	      Call GetU(t,rc) !solve for c,N,pi,and gamma, results are stored in u(t,1:4)
	      if (rc .ne. 0) then
	        FError=1
	        return
	      endif
	    
	     x(t+1,1)=(1./a)* (x(t,4)*(u(t,2)**alpha)*(x(t,1)**(1.-alpha))
     +                    +(1.-delta)*x(t,1) - u(t,1) )  !k(t+1)
	     if (x(t+1,1) .le. Epsilon(a)) then
	       FError=1
	       return
	     endif
	     x(t+1,2)=(x(t,5)*x(t,2))/(a*u(t,3)) !m(t+1)
	     if (x(t+1,2) .lt. Epsilon(a)) then
	       FError=1
	       return
 	     endif
	
	end do

      SSE=0.0

     	do t=ta,nrx-2

        temp(1)=beta*(a**(-eta))*lq(t+1,1)*
     +         (1.-delta+(1.-alpha)*x(t+1,4)*(u(t+1,2)**alpha)
     +          *(x(t+1,1)**(-alpha))) - PSI(Gamma(1:ng1),x(t,1:ncx))
        temp(2)=beta*(a**(-eta))*lq(t+1,1)*lq(t+1,2)*lq(t,2)	  
     +          *(1./u(t+1,3)) - PSI(Gamma(ng1+1:2*ng1),x(t,1:ncx))
	  temp(3)=beta*(a**(-eta))*(x(t+1,3))*(lq(t+1,1)+u(t+1,4))
     +          *(1./u(t+1,3)) - PSI(Gamma(2*ng1+1:3*ng1),x(t,1:ncx))
	  
        SSE(1)=SSE(1)+(temp(1)**2)
        SSE(2)=SSE(2)+(temp(2)**2)
	  SSE(3)=SSE(3)+(temp(3)**2)
      
	end do

      Do t=1,3
	 SSE(t)=dble(SSE(t)/nobs)
	End do

	Return
	End Subroutine

