C Ramsey3c.for:
C
C Purpose: Sovle the stochastic growth model using the
C          parameterized expectations approach.

C Module definitions
      Module Def1
	
	Real(8) alpha, beta, eta, delta, rho, sigma
	Integer(4) nobs, ta, dg, ng, nrx, ncx
	Real(8), allocatable :: phi(:), x(:,:), u(:), eps(:), lm(:)

	End Module Def1

C Here begins the main program
      program Ramsey3c

      use Def1

      use DFLIB
	use IMSLF90

      Integer(4) t, itn, rc, steps1, steps2, lags, freq(3), nofs, length
	Integer(2) row
	Real(4) Asec
      Real(8), Allocatable :: Gamma0(:), Gamma1(:), Gamma2(:), 
     +                        typx(:)      
	Real(8) ConTest1, converge, sse, TypS, StDev,  Lambda, chi(2)
	Real(8) stop1, stop2, eta0, eta1, delta0, delta1, incr, dm

	Character(256) Laufzeit
	Character(50) MyDate

	Logical initialize, ok

      External F, JacF

	Type (rccoord) curpos		

C Read Parameters from input file and compute derived parameters
C Parameters of the algorithm
      Call GetL('initialize','Parameters.txt',initialize)
	Call GetD('lambda','Parameters.txt',lambda)
	Call GetD('stop1' ,'Parameters.txt',stop1)
	Call GetD('stop2' ,'Parameters.txt',stop2)
	Call GetD('typx'  ,'Parameters.txt',converge)	
	Call GetD('typS'  ,'Parameters.txt',typS)
      Call GetI('nobs' ,'Parameters.txt',nobs )
	Call GetI('ta'   ,'Parameters.txt',ta   )
      Call GetI('dg'   ,'Parameters.txt',dg   )
	Call GetI('steps1','Parameters.txt',steps1)
	Call GetI('steps2','Parameters.txt',steps2)
	Call GetI('lags'  ,'Parameters.txt',lags)
	Call GetI('nofs'  ,'Parameters.txt',nofs)
	Call GetI('length','Parameters.txt',length)

C Parameters of the model
	Call GetD('alpha' ,'Parameters.txt',alpha)
	Call GetD('beta'  ,'Parameters.txt',beta )
	Call GetD('eta0'  ,'Parameters.txt',eta0 )
	Call GetD('eta1'  ,'Parameters.txt',eta1 ) 
	Call GetD('delta0','Parameters.txt',delta0)
	Call GetD('delta1','Parameters.txt',delta1)
	Call GetD('rho'  ,'Parameters.txt',rho  )
	Call GetD('sigma','Parameters.txt',sigma)

      ng=((dg+1)*(dg+2))/2
	nrx=nobs+ta+1

C Assign hard wired parameter values      	
	ncx=2
	
      
C Allocate space to u, x, and phi
      Allocate(u(1:nrx-1), x(1:nrx,1:ncx), phi(1:nobs),eps(1:nrx))	 
	Allocate(lm(1:nrx-1))
      Allocate(Gamma0(1:ng), Gamma1(1:ng), Gamma2(1:ng), typx(1:ng))
	
	typx=converge      

C Prepare output window
      ok=InitialSettings()
      Call MakeSmallWindow(10,"Program Progress",2)

      
C Fix starting time
      Asec=Secnds(0.0)

C Initiatize shocks
      If (initialize) then ! compute shocks and save for later use
  	   Call SetTextPosition(1,1,curpos)
	   WRITE(10,"('Initialize Shocks')")
	   Call DRNNOR(nrx,eps) ! Draw shocks from a standard normal	
	   open(20, form='unformatted',file='eps.bin')
	   write(20) eps(1:nrx)
	   close(20)
	   Gamma0(1)=Dlog(1.0/(beta*(1-alpha*beta)))
	   Gamma0(2)=-alpha
	   Gamma0(3)=-1.0	   
	   do t=4,ng,1
	    Gamma0(t)=0.0
	   end do	   

	else ! read shocks from file
	   
	   open(20,form='unformatted',file='eps.bin')
	   read(20), eps(1:nrx)
	   close(20)
  	   open(20,file='ParIni.txt')
	   do t=1,ng
	     read(20,"(F10.4)") Gamma0(t)
         end do	   
         close(20)
	endif

C Compute sequence of productivity shocks
      X(1,2)=exp(Sigma*eps(1))
      Do t=1,nrx-1
	      X(t+1,2)=(X(t,2)**Rho)*exp(Sigma*eps(t))
	End Do

      Gamma1=Gamma0
      open(15,file='LogFile.txt')

C      Gamma2(1)=1.9509
C      Gamma2(2)= -1.0961
C      Gamma2(3)= -1.3389
C      Gamma2(4)= 0.0379
C      Gamma2(5)= 0.2753
C      Gamma2(6)= -0.0806
C      delta=delta1
C      eta=eta1
C      rc=0
C	goto 7

C Here begin the iterations
      eta=eta0
	if (steps1 .eq. 0) goto 99

      incr=(delta0-delta1)/steps1
      if (incr .eq. 0.0) goto 99
	
      
      Do itn=1,steps1,1
	   
	   delta=delta0-dble(itn*incr)
	   Call SetTextPosition(1,2,curpos)
	   Write(10,"('Delta=',F7.4)") delta
	   write(15,"('delta=',F7.4)") delta
         Call GetGamma(Gamma1,typx,typs,stop1,lambda,Gamma2,rc)
	   if (rc .ne. 0) goto 100	   	   
         Write(15,"('Old parameter vector:   New parameter vector:')")
	   Do t=1,ng,1
	    Write(15,"(F8.4,14X,F8.4)") Gamma1(t), Gamma2(t)
	   End Do
	   Gamma1=Gamma2
	   
	End Do

	Gamma1=Gamma2
99    delta=delta1

      if (steps2 .eq. 0) then
	   eta=eta1
	   goto  101
	end if

	incr=(eta0-eta1)/steps2
      if (incr .eq. 0.0) goto 101

	Do itn=1,steps2
	   eta=eta0-dble(itn*incr)
         Call SetTextPosition(1,2,curpos)
         Write(10,"('eta  =',F7.4)") eta
	   write(15,"('eta=',F7.4)") eta
         Call GetGamma(Gamma1,typx,typs,stop1,lambda,Gamma2,rc)
	   if (rc .ne. 0) goto 100
         Write(15,"('Old parameter vector:   New parameter vector:')")
	   Do t=1,ng,1
	    Write(15,"(F8.4,14X,F8.4)") Gamma1(t), Gamma2(t)
	   End Do
	   Gamma1=Gamma2
	End Do

C Refinement of the Solution
      Gamma1=Gamma2
101   Call GetGamma(Gamma1,typx,typs,stop2,lambda,Gamma2,rc)

100	Call ElapsedTime(Secnds(Asec),Laufzeit)
      Call SetTextPosition(3,3,curpos)
	if (rc .eq. 0) then
	  Write(10,"('Converged',A)") Laufzeit
	else
	  Call SetTextPosition(4,3,curpos)
	  Write(10,"(A)") Laufzeit
	endif
	Call SetTextPosition(5,2,curpos)
	Write(10,"('Old Vector')")
	Call SetTextPosition(5,17,curpos)
	Write(10,"('New Vector')")
      row=6
	Do t=1,ng
        Call SetTextPosition(row,2,curpos)
	  Write(10,"(F10.4,2X,F10.4)") Gamma0(t), Gamma2(t)
	  row=row+1
	end do
      
	Call SetTextPosition(row+1,2,curpos)
	TypS=SSE(Gamma2)
	Write(10,"('Sum of Squared Errors=',F20.15)") TypS

	
	Call SetTextPosition(7,30,curpos)
	Write(10,"('Overwrite ParIni.txt: (T/F)?')")
	Call SetTextPosition(7,63,curpos)
	Read(10,"(L1)") ok
	if (ok) then
	  open(20,file='ParIni.txt')
	  Do t=1,ng
	    Write(20,"(F10.4))") Gamma2(t)
	  End Do
	  close(20)
	endif

	open(20, file='Summary.txt')
	Write(20,"(A50)") MyDate()
	Write(20,"('Run time: ',A)") Laufzeit
	if (rc .eq. 0) Write(20,"('Converged')")
	Write(20,"('Sum of squared errors:            ',F20.15)") TypS
	Write(20,"('Standard deviation of consumption:',F10.6)")
     +        StDev(nobs,u(ta:nrx-1))
	Write(20,"('Standard deviation of capital    :',F10.6)")
     +        StDev(nobs,x(ta:nrx-2,1))
      Write(20,"('Standard deviation of shock      :',F10.6)")
     +        StDev(nobs,x(ta:nrx-2,2))		
	if (rc .eq. 0)
     + 	Write(20,"('Number of binding constraints    :',I6)") itn
	Write(20,"('Old parameter vector:   New parameter vector:')")
	  Do t=1,ng,1
	   Write(20,"(F8.4,14X,F8.4)") Gamma0(t), Gamma2(t)
	End Do

	if (rc. eq. 0) then
	  itn=Count(lm .gt. 0.0)
	  Deallocate(phi)
        chi(1)=0.025
	  chi(2)=0.975
        Call DMStat(lags,length,dm,chi)
	  Call Simulate(nofs,lags,length,freq,ng,Gamma2)	

	   Write(20,"('DM-Stat=',F10.3,' Chi(2.5)=',F7.3,' Chi(97.5)=',
     +         F7.3)") DM, Chi(1), Chi(2)
	   Write(20,"('Total valid Simulations:',I5)") freq(3)
	   Write(20,"('in the lower tail      :',I5)") freq(1)
	   Write(20,"('in the upper tail      :',I5)") freq(2)
	   Write(20,"('Percentage in the lower tail:',F6.2)")
     +     100*dble(freq(1))/dble(freq(3))
	   Write(20,"('Percentage in the upper tail:',F6.2)")
     +     100*dble(freq(2))/dble(freq(3))
	endif

      Write(20,"('Parameters of the model:')")
	Write(20,"('alpha =  ',F8.4)") alpha
	Write(20,"('beta  =  ',F8.4)") beta
	Write(20,"('eta   =  ',F8.4)") eta
	Write(20,"('delta =  ',F8.4)") delta
	Write(20,"('rho   =  ',F8.4)") rho
	Write(20,"('sigma =  ',F8.4)") sigma
	Write(20,"('Parameters of the simulations:')")
	Write(20,"('degree=  ',I6)") dg
	Write(20,"('nobs  =  ',I6)") nobs
      Write(20,"('lags  =  ',I6)") lags	
	Write(20,"('nofs  =  ',I6)") nofs
	Write(20,"('length=  ',I6)") length
	

	close(20)
	
	End


C **************************  Subroutines ********************************
C
C
C
C GetGamma: Computes the fixed point for a given set of parameters

      Subroutine GetGamma(Gamma0,typx,typs,stop1,lambda,Gamma2,rc)

	use Def1
	use DFLIB

      Integer(4) itn, maxit, rc
	Real(8) Gamma0(ng), Gamma1(ng), Gamma2(ng), typx(ng), typs, stop1,
     +        lambda, converge      

	External F, JacF

	Type (rccoord) curpos		

	maxit=5000
	Gamma1=Gamma0
      
      Call SetTextPosition(2,60,curpos)
	Write(10,"(A10)") "          "
	
	Do itn=1,maxit
	 
	 Call SetTextPosition(2,2,curpos)
	 Write(10,"(A,I4)") "Iteration No. ", itn	 
	 rc=0
       Call GetPhi(Gamma1,rc)

	 Call GaussNewton(ng,nobs,Gamma1,typx,typS,F,JacF,Gamma2,rc)
	 if (rc .ne. 0) then
	  Call SetTextPosition(3,3,curpos)
	  Select Case(rc)
	   Case(1)
	    Write(10,"('GN: failure to solve for Gamma')")
	    Write(15,"('GN: failure to solve for Gamma')")
	   Case(2)
          Write(10,"('GN: minimal step size without convergence')")
	    Write(15,"('GN: minimal step size without convergence')")
	   Case(3)
          Write(10,"('GN: convergence to non-opitmal point')")
	    Write(15,"('GN: convergence to non-opitmal point')")
	   Case(4)
  	    Write(10,"('GN: maximum number of iterations exceeded')")
	    Write(15,"('GN: maximum number of iterations exceeded')")
        End Select
	  Exit
	 Endif
       converge=ConTest1(ng,Gamma1,Gamma2,typx)
	 Call SetTextPosition(2,25,curpos)
	 Write(10,"('ConTest1=',F20.15)") converge
	 if (converge .lt. stop1) then
	    Call SetTextPosition(2,60,curpos)
	    Write(10,"('Converged!')")
		exit
	 endif
	 Gamma1=lambda*Gamma2+(1-lambda)*Gamma1       

	End Do

	Gamma2=Gamma1
	Write(15,"('Iterations: ',I4)") itn
	Write(15,"('Parameter convergence criterim:   ',F12.7)") converge
	converge=SSE(Gamma2)
	Write(15,"('Sum of squared errors:            ',F20.15)") converge
	Write(15,"('Standard deviation of consumption:',F10.6)")
     +        StDev(nobs,u(ta:nrx-1))

	Return

	End Subroutine

C Psi returns Psi(Gamma,x(t))
C
      Real(8) Function Psi(gamma,xt)

	use Def1

      Real(8) gamma(ng), xt(ncx)

	Integer(4) j, j1, gi
	Real(8) temp
      Psi=gamma(1)
	gi=1
	do j=1,dg
	   do j1=0,j
            gi=gi+1
	      temp=(dlog(xt(1))**(j-j1))*(dlog(xt(2))**j1)
	      Psi=Psi+gamma(gi)*temp
	   end do
	end do

      Psi=beta*exp(Psi)

	Return
	End Function

C F returns the vector of errors

      Subroutine F(m,n,Gamma,err)

      use Def1
  
      Integer(4) m, n
	Real(8) gamma(m), err(n)

	Integer(4) t
	External PSI

	do t=ta,nrx-2
          
	    err(t-ta+1)=phi(t-ta+1)-PSI(Gamma,x(t,1:ncx))

	end do

	Return

	End Subroutine



C SSE: returns the sum of squared errors

      Real(8) Function SSE(Gamma)

      use Def1
	Real(8) Gamma(ng)

      Real(8) err(nobs)

      External F

	Call F(ng,nobs,Gamma,err)

      SSE=Dot_product(err,err)
	SSE=dble(SSE/nobs)

	Return

	End Function

	
C JacF returns the Jacobian of F
      Subroutine JacF(m,n,gamma,xx)

	use Def1

	Integer(4) m, n
	Real(8) gamma(m), xx(1:n,1:m)

	Integer(4) t,j,j1,gi
	Real(8) temp
	
	External Psi
	
      
      Do t=ta,nrx-2
	   gi=1
         xx(t-ta+1,1)=PSI(Gamma,X(t,1:ncx))
	   Do j=1,dg
	      Do j1=0,j
	         gi=gi+1
               temp=(dlog(x(t,1))**(j-j1))*(dlog(x(t,2))**j1)
	         xx(t-ta+1,gi)=xx(t-ta+1,1)*temp
	      End Do
	   End Do

      End Do

	Return
	End Subroutine
	         

C GetPhi: Simulates the model to get Phi and X(t,1)

      Subroutine GetPhi(Gamma,rc)

      use Def1

	Integer(4) rc
	Real(8) Gamma(ng)
    
      Integer(4) t
	Real(8) rhs
      
	rc=0

	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))

	do t=1,nrx-1

	       rhs=PSI(Gamma,x(t,1:ncx))
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif	  

	end do

	do t=ta,nrx-2
	   phi(t-ta+1)=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta)
	end do

	Return

	End Subroutine

C DMStat
      Subroutine DMStat(l,length,dm,chi)
                       
	use Def1
	use IMSLF90

	Integer(4) l, length
	Real(8) DM(1,1), Chi(1:2), p(1:2)

	Integer(4) t, t1, j, m
	Real(8) errvec(1:length,1), z(1:length,1+2*l), Amat(1:2*l+1,1:2*l+1),
     +        Bvec(1:2*l+1,1), Avec(1:2*l+1,1), Help(1:2*l+1,1), DCHIIN

      m=1+2*l
      t1=length+ta+l-1

      Do t=ta+l,t1
         
	   errvec(t-ta-l+1,1)=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +                   (x(t+1,1)**(alpha-1)))- beta*lm(t+1)*(1-delta)
     +    - (u(t)**(-eta)) + lm(t)
         z(t-ta-l+1,1)=1.0
	   Do j=1,l
	      z(t-ta-l+1,1+j)=u(t-j)
	      z(t-ta-l+1,1+l+j)=x(t-j,2)
	   End Do

	End Do

      Avec=MatMul(Transpose(z),errvec)
      Amat=0
	Do t=1,length
	   Help(1:m,1)=z(t,1:m)
	   Amat(1:m,1:m)=Amat(1:m,1:m)+(errvec(t,1)**2)*
     +   MatMul(Help,Transpose(Help))
	End Do

	Call DLINRG(m,Amat,m,Amat,m)

      Bvec=MatMul(Amat,Avec)
	DM=MatMul(Transpose(Avec),Bvec)
     
	p(1)=chi(1)
	p(2)=chi(2)
      
	chi(1)=DCHIIN(p(1),dble(2*l+1))
	chi(2)=DCHIIN(p(2),dble(2*l+1))

	Return

	End Subroutine

      
      Subroutine Simulate(nofs,lags,length,freq,m,Gamma)
	                         
      use Def1

	Integer(4) nofs, lags, freq(3), m, length
      Real(8) Gamma(m), Psi

      Integer(4) t, i, t1
	Real(8) rhs, dm, chi(2)

	External Psi

	freq=0
	t1=length+ta+lags

C Loop over number of simulations
      Do i=1, nofs,1

C Draw shocks
	   Call DRNNOR(t1+1,eps) ! Draw shocks from a standard normal
	   X(1,2)=exp(Sigma*eps(1))
         Do t=1,t1
	      X(t+1,2)=(X(t,2)**Rho)*exp(Sigma*eps(t))
	   End Do


C Generate Time Series

      x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
      do t=1,t1

	       rhs=PSI(Gamma,x(t,1:ncx))	       
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

	end do	
      freq(3)=freq(3)+1

      chi(1)=0.05
	chi(2)=0.95

      Call DMStat(lags,length,dm,chi)

      if (dm .lt. chi(1)) freq(1)=freq(1)+1
	if (dm .gt. chi(2)) freq(2)=freq(2)+1

2     continue

      End Do

	Return

	End Subroutine
      

       