C Ramsey3c3.for:
C
C Purpose: Sovle the stochastic growth model with non-negative investment using the
C          parameterized expectations approach.
C          The solution is computed from the solution to
C          a system of non-linear equations.
C          Genetic search is used to find good starting values.


C Module definitions
      Module Def1
	
	Real(8) alpha, beta, eta, delta, rho, sigma
	Integer(4) nobs, ta, mod, ng, nrx, ncx
	Real(8) MaxExp1, MaxExp2, MinExp1
	Real(8), allocatable :: x(:,:), u(:), eps(:), lm(:)      
      Integer(4) Search, PsiError(2)

	End Module Def1

C Here begins the main program
      program Ramsey3c3

      use DFLIB
	use IMSLF90
	
      use Def1
	use MNR
	

      Integer(4) t, rc, lags, nums, maxs, nofs, freq(3), length
	Integer(2) row
	Real(4) Asec
      Real(8), Allocatable :: Gamma1(:), Gamma2(:)
	Real(8) StDev, dm, chi(2), fit, SSE, crit(1:4)
	Real(8) GetSSE

	Character(256) Laufzeit
	Character(50) MyDate

	Logical initialize, ok, InitialSettings

      External Fit1a, Fit1b, Fit2a, Fit2b, Fit3a, Fit3b, Fit4a, Fit4b
	External Fit5a, Fit5b
      External Psi1, Psi2, Psi3, dPsi1, dPsi2, dPsi3, Psi4, dPsi4
	External Psi5, dPsi5

	Type (rccoord) curpos		

C Read Parameters from input file and compute derived parameters
C Parameters of the algorithm
      Call GetL('initialize','Parameters.txt',initialize)
	Call GetI('nobs' ,'Parameters.txt',nobs )
	Call GetI('ta'   ,'Parameters.txt',ta   )
      Call GetI('mod'   ,'Parameters.txt',mod )
	Call GetI('lags'  ,'Parameters.txt',lags)
	Call GetI('search','Parameters.txt',search)
	Call GetI('nofs'  ,'Parameters.txt',nofs)
	Call GetI('length','Parameters.txt',length)

C Parameters of the model
	Call GetD('alpha' ,'Parameters.txt',alpha)
	Call GetD('beta'  ,'Parameters.txt',beta )
	Call GetD('eta'  ,'Parameters.txt',eta ) 
	Call GetD('delta','Parameters.txt',delta)	
	Call GetD('rho'  ,'Parameters.txt',rho  )
	Call GetD('sigma','Parameters.txt',sigma)

      Select Case(mod)
	   Case(1)
	    ng=3
	   Case(2)
	    ng=6
	   Case(3)
	    ng=7
         Case(4)
	    ng=7
	   Case(5)
	    ng=4
	End Select 
	
	nrx=nobs+ta+1
	MaxExp1=2.0
	MaxExp2=10.0
      MaxExp1=dlog(MaxExp1)
	MaxExp2=dlog(MaxExp2)
	MinExp1=MaxExp1/MaxExp2
	t=MaxExponent(MinExp1)

	MaxExp2=0.95*dble(t*MinExp1)
	MaxExp2=0.5*MaxExp2
	MaxExp1=0.95*dble(t*MaxExp1)
	t=MinExponent(MinExp1)
	MinExp1=2.0
	MinExp1=dlog(MinExp1)
	MinExp1=0.95*dble(t*MinExp1)


C Assign hard wired parameter values      	
	ncx=2
	
      
C Allocate space to u, x, and phi
      Allocate(u(1:nrx-1), lm(1:nrx-1), x(1:nrx,1:ncx))	 
      Allocate(Gamma1(1:ng), Gamma2(1:ng))
	Allocate(eps(1:nrx))

C Prepare output window
      ok=InitialSettings()
      Call MakeSmallWindow(10,"Program Progress",2)

      
C Fix starting time
      Asec=Secnds(0.0)
      

      open(11,file='LogFile.txt')	
	open(15,file='Summary.txt')

	WRITE(11,"(A50)") MyDate()	
	WRITE(15,"(A50)") MyDate()

C Initiatize shocks
      If (initialize) then ! compute shocks and save for later use
  	   Call SetTextPosition(1,1,curpos)
	   WRITE(10,"('Initialize Shocks')")	   
	   Call DRNNOR(nrx,eps) ! Draw shocks from a standard normal
         open(20, form='unformatted',file='eps.bin')
	   write(20) eps(1:nrx)
	   close(20)
	   Call SetTextPosition(1,1,curpos)
	   Write(10,"('                   ')")	   
	
	else ! read shocks from file
	   
	   open(20,form='unformatted',file='eps.bin')
	   read(20), eps(1:nrx)
	   close(20)
  	
	endif

C Compute sequence of productivity shocks
      X(1,2)=exp(Sigma*eps(1))
      Do t=1,nrx-1
	   X(t+1,2)=(X(t,2)**Rho)*exp(Sigma*eps(t))
	End Do

C Start with given inital values
      if (search .eq. 0) then
	  open(20,file='ParIni.txt')
	  do t=1,ng
	       read(20,"(F10.4)") Gamma1(t)
        end do	   
        close(20)	            	      			   
	  Select Case(mod)  ! find solution
          Case(1)
		 Call FixvMN(ng,Gamma1,Fit1a,Gamma2,rc,crit)	      
	    Case(2)
	      Call FixvMN(ng,Gamma1,Fit2a,Gamma2,rc,crit)	      
	    Case(3)
	      Call FixvMN(ng,Gamma1,Fit3a,Gamma2,rc,crit)	      
         Case(4)		      
	      Call FixvMN(ng,Gamma1,Fit4a,Gamma2,rc,crit)	      
	   Case(5)		      
	      Call FixvMN(ng,Gamma1,Fit5a,Gamma2,rc,crit)	      
	  End Select	
        goto 7
	endif

C or find good starting values
      maxs=10      

C Try to find solution
      Do nums=1,maxs           
        Select Case(mod) ! use GSearch to find good initial value
	     Case(1) 	  
             if (search .eq. 1) Call GSearch1(Fit1b,ng,Gamma1,Fit)
	       if (search .eq. 2) Call GSearch2(Fit1b,ng,Gamma1,Fit)
	     Case(2)
	       if (search .eq. 1) Call GSearch1(Fit2b,ng,Gamma1,Fit)
	       if (search .eq. 2) Call GSearch2(Fit2b,ng,Gamma1,Fit)
	     Case(3)
	       if (search .eq. 1) Call GSearch1(Fit3b,ng,Gamma1,Fit)
	       if (search .eq. 2) Call GSearch2(Fit3b,ng,Gamma1,Fit)
	     Case(4)
	       if (search .eq. 1) Call GSearch1(Fit4b,ng,Gamma1,Fit)
             if (search .eq. 2) Call GSearch2(Fit4b,ng,Gamma1,Fit)
	     Case(5)
	       if (search .eq. 1) Call GSearch1(Fit5b,ng,Gamma1,Fit)
	       if (search .eq. 2) Call GSearch2(Fit5b,ng,Gamma1,Fit)
	   End Select
     	   Call ClearScreen($GCLEARSCREEN)        
	   CALL SETTEXTPOSITION(1,2,curpos)
	   WRITE(10,"('Simulation No. ',I3)") nums
	   WRITE(11,"('Simulation No. ',I3)") nums	
	   CALL SETTEXTPOSITION(2,2,curpos)
	   WRITE(10,"('Solution from GSearch with Fit=',1X,F15.8)") Fit
	   WRITE(11,"('Solution from GSearch with Fit=',1X,F15.8)") Fit	
	   CALL SETTEXTPOSITION(3,2,curpos)
	   WRITE(10,*) 'Found Vector:'
	   WRITE(11,*) 'Found Vector:'
	   row=4		
	   DO t=1,ng
	     CALL SETTEXTPOSITION(row,2,curpos)	   
	     WRITE(10,"(F12.8)") Gamma1(t)
	     WRITE(11,"(F12.8)") Gamma1(t)	     
	     row=row+1 
	    END DO
        Select Case(mod)  ! refine solution
          Case(1)
		 Call FixvMN(ng,Gamma1,Fit1a,Gamma2,rc,crit)	      
	    Case(2)
	      Call FixvMN(ng,Gamma1,Fit2a,Gamma2,rc,crit)	      
	    Case(3)
	      Call FixvMN(ng,Gamma1,Fit3a,Gamma2,rc,crit)	      
         Case(4)		      
	      Call FixvMN(ng,Gamma1,Fit4a,Gamma2,rc,crit)	      
	   Case(5)
	      Call FixvMN(ng,Gamma1,Fit5a,Gamma2,rc,crit)
	  End Select	
	  Call SetTextPosition(row+1,2,curpos)
	if (rc .eq. 1) then
	  WRITE(10,"('Error Message from FixvMN: ',A,' Psi:',2I)")
     +   MNR_Message(rc), PsiError(1:2)
	  WRITE(11,"('Error Message from FixvMN: ',A,' Psi:',2I)")
     + MNR_Message(rc), PsiError(1:2)
      else
         WRITE(10,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	   WRITE(11,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	endif
	  Do t=1,ng
         Call SetTextPosition(row,2,curpos)	    
	   Write(11,"(F10.4)") Gamma2(t)
	   row=row+1
	  End do             
	  
        if (rc .eq. 0) exit
	 
	End Do
	
      
7     Call ElapsedTime(Secnds(Asec),Laufzeit)      
	Write(15,"('Run time=',A)") Laufzeit
	if (nums .gt. maxs) then
	 Call SetTextPosition(row+1,2,curpos)
	 WRITE(10,"('Not able to find solution after',I3,' Iterations')")
     +  maxs
      else
	  Write(15,"('Error Message from FixVMN',A)") MNR_Message(rc)
	  Write(15,"('abs(f(i))=',E15.7)") crit(1)
	  Write(15,"('abs(dx/x)=',E15.7)") crit(2)
	  Write(15,"('GradTest =',E15.7)") crit(3)
	  Write(15,"('0.5*f*f  =',E15.7)") crit(4)      
	  Write(15,"('Final Solution')")
        Do t=1,ng         
	   Write(15,"(F10.4)") Gamma2(t)
	   row=row+1
	  End do             
	  if (rc .eq. 0) then
         Select Case(mod)
	    Case(1)
	     SSE=GetSSE(Gamma2,Psi1)
	     Call Tests(Gamma2,Psi1,20)	     
	    Case(2)	      
	      SSE=GetSSE(Gamma2,Psi2)
	    Case(3)
	     SSE=GetSSE(Gamma2,Psi3)
	    Case(5)
	     SSE=GetSSE(Gamma2,Psi5)
	   End Select
	   Write(15,"('Sum of squared errors:            ',F20.15)") SSE     
	   chi(1)=0.025
	   chi(2)=0.975
         Call DMStat(lags,length,dm,chi)	      			   
	   Write(15,"('Standard deviation of consumption:',F10.6)")
     +        StDev(nobs,u(ta:nrx-1))
	   Write(15,"('Standard deviation of capital    :',F10.6)")
     +        StDev(nobs,x(ta:nrx-2,1))
         Write(15,"('Standard deviation of shock      :',F10.6)")
     +        StDev(nobs,x(ta:nrx-2,2))
	   Write(15,"('Number of binding constraints:',I5)")
     +    Count(lm .gt. 0.0)
	  Write(15,"('DM-Stat=',F10.3,' Chi(2.5)=',F7.3,' Chi(97.5)=',
     +         F7.3)") DM, Chi(1), Chi(2)	
	  Select Case(mod)
	    Case(1)
		 Call Simulate(nofs,lags,length,freq,ng,Gamma2,Psi1)
	    Case(2)
           Call Simulate(nofs,lags,length,freq,ng,Gamma2,Psi2)
	    Case(3)
	     Call Simulate(nofs,lags,length,freq,ng,Gamma2,Psi3)
	    Case(4)
	     Call Simulate(nofs,lags,length,freq,ng,Gamma2,Psi4)
	    Case(5)
           Call Simulate(nofs,lags,length,freq,ng,Gamma2,Psi5)
	   End Select
         Write(15,"('Total valid Simulations:',I5)") freq(3)
	   Write(15,"('in the lower tail      :',I5)") freq(1)
	   Write(15,"('in the upper tail      :',I5)") freq(2)
	   Write(15,"('Percentage in the lower tail:',F6.2)")
     +     100*dble(freq(1))/dble(freq(3))
	   Write(15,"('Percentage in the upper tail:',F6.2)")
     +     100*dble(freq(2))/dble(freq(3))	   
	   Write(15,"('Number of binding constraints:',I5)")
     +    Count(lm .gt. 0.0)
       endif
	endif
         Write(15,"('Parameters of the model:')")
	   Write(15,"('alpha=  ',F7.3)") alpha
	   Write(15,"('beta =  ',F7.3)") beta
	   Write(15,"('eta  =  ',F7.3)") eta
	   Write(15,"('delta=  ',F7.3)") delta
	   Write(15,"('rho  =  ',F7.3)") rho
	   Write(15,"('sigma=  ',F7.4)") sigma
	   Write(15,"('Parameters of the simulations:')")
	   Write(15,"('psi =  ',I2)") mod
	   Write(15,"('nobs=  ',I6)") nobs
         Write(15,"('lags=  ',I6)") lags
	   Write(15,"('nobs=  ',I6)") nobs
	   Write(15,"('nofs=  ',I8)") nofs
	   Write(15,"('length=',I8)") length
	close(11)
	close(15)	         
      
	End


C **************************  Subroutines ********************************
C
C
C
C Psi1 returns Psi(Gamma,x(t)) in the case of a first degree polynomial
      Real(8) Function Psi1(gamma,xt)

	use Def1
	use MNR

      Real(8) gamma(ng), xt(ncx)
      
	Psi1=gamma(1) + gamma(2)*dlog(xt(1)) + gamma(3)*dlog(xt(2))
      PsiError=0
      If (Psi1 .gt. MaxExp1) then
	   FError=1
	   PsiError(1)=1
	   return
C	else if (Psi1 .lt. MinExp1) then	 
C	   FError=1
C	   PsiError(2)=1
C	   return
	else
	   Psi1=beta*exp(Psi1)
	   if (Psi1 .le. 0.0) FError=1
	endif      

	Return
	End Function

C dPsi1: computes the gradient of Psi1 in the case of a first degree polynomial
      Subroutine dPsi1(Gamma,xt,g)

      use Def1
      use MNR

	Real(8) Gamma(ng), xt(1:ncx), g(ng), Psi1

      External Psi1

         g(1)=Psi1(Gamma,xt)
	   g(2)=g(1)*dlog(xt(1))
	   g(3)=g(1)*dlog(xt(2))

      Return

	End Subroutine

 

C Psi2 returns Psi(Gamma,x(t)) in the case of a second degree polynomial
C
      Real(8) Function Psi2(gamma,xt)

	use Def1
	use MNR

      Real(8) gamma(ng), xt(ncx)

      PsiError=0

	Psi2=gamma(1) + gamma(2)*dlog(xt(1)) + gamma(3)*dlog(xt(2)) +
     +     gamma(4)*(dlog(xt(1))**2) + gamma(5)*dlog(xt(1))*dlog(xt(2))+
     +     gamma(6)*(dlog(xt(2))**2)

      If (Psi2 .gt. MaxExp1) then	  
	   FError=1
	   PsiError(1)=1
	   return
	else if (Psi2 .lt. MinExp1) then	   
	   FError=1
	   PsiError(2)=1
	   return
	else
	   Psi2=beta*exp(Psi2)
 	endif

	Return
	End Function


C dPsi2: computes the gradient of Psi2 in the case of a second degree polynomial
      
	Subroutine dPsi2(Gamma,xt,g)

      use Def1
      use MNR

	Real(8) Gamma(ng), xt(1:ncx), g(ng), Psi2

      External Psi2

         g(1)=Psi2(Gamma,xt)
	   g(2)=g(1)*dlog(xt(1))
	   g(3)=g(1)*dlog(xt(2))
	   g(4)=g(1)*dlog(xt(1))*dlog(xt(1))
	   g(5)=g(1)*dlog(xt(1))*dlog(xt(2))
         g(6)=g(1)*dlog(xt(2))*dlog(xt(2))

      Return

	End Subroutine


C Psi3: Computes Psi(Gamma,xt) in the case of a neural network (exponential form)
      Real(8) Function Psi3(Gamma,xt)

	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx)

      Real(8) temp1, temp2, arg1, arg2

         arg1=-gamma(3)*xt(1)-gamma(4)*xt(2)

      PsiError=0

	if (arg1 .gt. MaxExp1) then
	   FError=1
	   PsiError(1)=1
	   return
	else if (arg1 .lt. MinExp1) then
	   FError=1
	   PsiError(2)=1
	   return
	else
	   temp1=1.0/(1.0 + exp(arg1))
	endif
         
	   arg2=-gamma(5)*xt(1)-gamma(6)*xt(2)

	   if (arg2 .gt. MaxExp1) then
	      FError=1
	      PsiError(1)=1
	      return
	  elseif (arg2 .lt. MinExp1) then	      
	      FError=1
	      PsiError(2)=1
	      return
       else 
            temp2=1.0/(1.0+exp(arg2))
       endif

      
	Psi3=gamma(1) + (gamma(2)*temp1) + gamma(7)*temp2
      
	if (Psi3 .gt. MaxExp1) then	
	    FError=1
	    PsiError(1)=1
	    return
	else if (Psi3 .lt. MinExp1) then	    
	    FError=1
	    PsiError(2)=1
          return
	else
	  Psi3=beta*exp(Psi3)
	endif
	Return
	End Function


C Psi4: Computes Psi(Gamma,xt) in the case of a neural network
      Real(8) Function Psi4(Gamma,xt)

	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx)

      Real(8) arg, temp1, temp2

         arg=-gamma(3)*xt(1)-gamma(4)*xt(2)
	   PsiError=0
         if (arg .gt. MaxExp1) then
	      FError=1
	      PsiError(1)=1
	      return
	   else if (arg .lt. MinExp1) then	      
	      FError=1
	      PsiError(2)=1
	      return
	   else
	      temp1=1.0/(1.0 + exp(arg))
	   endif

	   arg=-gamma(5)*xt(1)-gamma(6)*xt(2)
        
        if (arg .gt. MaxExp1) then	     
	     FError=1
	     PsiError(1)=1
	     return
	  else if  (arg .lt. MinExp1) then	     
	     FError=1
	     PsiError(2)=1
	     return
	  else
	    temp2=1.0/(1.0+exp(arg))
	  endif	   
        Psi4=gamma(1) + gamma(2)*temp1 + gamma(7)*temp2
	  Psi4=beta*Psi4
	  
	  if (Psi4 .lt. 0.0) FError=1
	
	Return
	End Function

C Psi5: Computes Psi(Gamma,xt) in the case of a neural network (exponential form) with
C       3 Parameters
      Real(8) Function Psi5(Gamma,xt)


	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx)

      Real(8) arg

         arg=-gamma(3)*xt(1)-gamma(4)*xt(2)

      PsiError=0

	if (arg .gt. MaxExp1) then
	   Psi5=gamma(1)
	elseif (arg .lt. MinExp1) then
	   Psi5=gamma(1)+gamma(2)
	else
	   Psi5=gamma(1) +(gamma(2)/(1.0 + exp(arg)))
	endif
         	
	Psi5=beta*Psi5
	
	if (Psi5 .lt. 0.0) FError=1

	Return
	End Function


C dPsi3 computes the gradient of Psi3
      Subroutine dPsi3(Gamma,xt,g)

	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx), g(ng)

      Real(8) arg, temp, Psi3

	External Psi3
      FError=0

	 g(1)=Psi3(Gamma,xt)

       if (FError .ne. 0) return

	 arg=-gamma(3)*xt(1)-gamma(4)*xt(2)
	 if (arg .gt. MaxExp1) then
	   g(2)=0.0
	   g(3)=0.0
	   g(4)=0.0
	 else if(arg .lt. MinExp1) then
	   g(2)=g(1)
	   g(3)=0.0
	   g(4)=0.0
	 else
	   temp=1.0+exp(arg)
	   g(2)=g(1)/temp
	   g(3)=g(1)*(gamma(2)/(temp**2.0))*(temp-1.0)*xt(1)
	   g(4)=g(1)*(gamma(2)/(temp**2.0))*(temp-1.0)*xt(2)
	 endif

	 arg=-gamma(5)*xt(1)-gamma(6)*xt(2)
	 if (arg .gt. MaxExp1) then
	   g(5)=0.0
	   g(6)=0.0
	   g(7)=0.0
	 else if(arg .lt. MinExp1) then
	   g(5)=0.0
	   g(6)=0.0
	   g(7)=g(1)
	 else
	   temp=1.0+exp(arg)
	   g(5)=g(1)*(gamma(7)/(temp**2.0))*(temp-1.0)*xt(1)
	   g(6)=g(1)*(gamma(7)/(temp**2.0))*(temp-1.0)*xt(2)	     
	   g(7)=g(1)/temp
	 endif
   
           
	Return
	End Subroutine

C dPsi4 computes the gradient of Psi4

      Subroutine dPsi4(Gamma,xt,g)

	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx), g(ng)

      Real(8) arg, temp	

	 g(1)=beta

       arg=-gamma(3)*xt(1)-gamma(4)*xt(2)

	    temp=1.0 + exp(arg)
	    g(2)=beta/temp
	    g(3)=beta*(gamma(2)*(temp-1)*xt(1))/(temp**2.0)
	    g(4)=beta*(gamma(2)*(temp-1)*xt(2))/(temp**2.0)


	 arg=-gamma(5)*xt(1)-gamma(6)*xt(2)

	    temp=1.0+exp(arg)
	    g(5)=(beta*gamma(7)*(temp-1)*xt(1))/(temp**2.0)
	    g(6)=(beta*gamma(7)*(temp-1)*xt(2))/(temp**2.0)
	    g(7)=beta/temp

	Return
	End Subroutine

C dPsi5 computes the gradient of Psi5

      Subroutine dPsi5(Gamma,xt,g)

	use Def1
      use MNR

      Real(8) gamma(ng), xt(ncx), g(ng)

      Real(8) arg, temp, Psi5

	External Psi5
      
	FError=0

	 g(1)=beta       

	 arg=-gamma(3)*xt(1)-gamma(4)*xt(2)
	 if ((arg .gt. MaxExp1) .or. (arg .lt. MinExp1)) then
	   g(2)=0.0
	   g(3)=0.0
	   g(4)=0.0	   
	 else
	   temp=1.0+exp(arg)
	   g(2)=beta/temp
	   g(3)=beta*gamma(2)*((temp-1.)*xt(1))/(temp**2.0)
         g(4)=beta*gamma(2)*((temp-1.)*xt(2))/(temp**2.0)
	 endif	 
           
	Return
	End Subroutine


C Fit1a: Computes the rhs of the system of equations, whose zero is the
C       PEA solution in the case of first degree polynomial

      Subroutine Fit1a(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j
	Real(8) rhs, temp, Psi1

	External Psi1
      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
	do t=1,nrx-1

	       rhs=PSI1(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) return
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)
	       if (x(t+1,1) .lt. Epsilon(rhs)) then
	          FError=1
	          return
	       endif

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif


	end do

      fit=0.0

	do t=ta,nrx-2

          rhs=PSI1(Gamma,x(t,1:ncx))
	   temp=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta) - rhs
	   	   
	   Call dPsi1(Gamma,x(t,1:ncx),g)

         do j=1,n
	      fit(j)=fit(j)+temp*g(j)
	   end do
	   	      
      End Do

      if (MaxVal(dabs(fit)) .gt. (10.**MaxExp2)) FError=1
	
	Return

	End Subroutine

C Fit2a: Computes the rhs of the system of equations, whose zero is the
C       PEA solution in the case of a second degree polynomial

      Subroutine Fit2a(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j
	Real(8) rhs, temp, Psi2

	External Psi2
      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
	do t=1,nrx-1

	       rhs=PSI2(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) return
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)
           if (x(t+1,1) .lt. Epsilon(rhs)) then
	          FError=1
	          return
	     endif
C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

	end do

      fit=0.0

	do t=ta,nrx-2

          rhs=PSI2(Gamma,x(t,1:ncx))
	   temp=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta) - rhs
	   	   
	   Call dPsi2(Gamma,x(t,1:ncx),g)

         do j=1,n
	      fit(j)=fit(j)+temp*g(j)
	   end do
	   	   
	   
      End Do
	if (MaxVal(dabs(fit)) .gt. (10.0**MaxExp2)) FError=1      
	Return

	End Subroutine

C Fit3a: Computes the rhs of the system of equations, whose zero is the
C        PEA solution in the case of a neural network (exponential form)

      Subroutine Fit3a(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j
	Real(8) rhs, temp, Psi3

      External Psi3
	      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
	fit=0.0
	
	do t=1,nrx-1

	       rhs=PSI3(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) return
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)
             if (x(t+1,1) .lt. Epsilon(rhs)) then
	          FError=1
	          return
	       endif
C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

	end do      

	do t=ta,nrx-2

          rhs=PSI3(Gamma,x(t,1:ncx))
	   temp=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta) - rhs
	   	   
	   Call dPsi3(Gamma,x(t,1:ncx),g)

         do j=1,n
	      fit(j)=fit(j)+temp*g(j)
	   end do
	   	   
	   
      End Do
	if (MaxVal(dabs(fit)) .gt. (10.0**MaxExp2)) FError=1      
	Return

	End Subroutine

C Fit4a: Computes the rhs of the system of equations, whose zero is the
C        PEA solution in the case of a neural network

      Subroutine Fit4a(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j
	Real(8) rhs, temp, Psi4

      External Psi4
	      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
	fit=0.0
	
	do t=1,nrx-1

	       rhs=PSI4(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) return
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)
             if (x(t+1,1) .lt. Epsilon(rhs)) then
	          FError=1
	          return
	       endif
C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif
	  
	end do      

	do t=ta,nrx-2

          rhs=PSI4(Gamma,x(t,1:ncx))
	   temp=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta) - rhs
	   	   
	   Call dPsi4(Gamma,x(t,1:ncx),g)

         do j=1,n
	      fit(j)=fit(j)+temp*g(j)
	   end do
	   	   
	   
      End Do
	if (MaxVal(dabs(fit)) .gt. (10.0**MaxExp2)) FError=1      
	Return

	End Subroutine


C Fit5a: Computes the rhs of the system of equations, whose zero is the
C       PEA solution in the case of neural network with 3 parameters

      Subroutine Fit5a(n,m,Gamma,fit)

      use Def1
	use MNR

	Integer(4) n, m
	Real(8) Gamma(m), fit(n), g(n)
    
      Integer(4) t, j
	Real(8) rhs, temp, Psi5

	External Psi5
      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
	do t=1,nrx-1

	       rhs=PSI5(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) return
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)
	       if (x(t+1,1) .lt. Epsilon(rhs)) then
	          FError=1
	          return
	       endif

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif


	end do

      fit=0.0

	do t=ta,nrx-2

          rhs=PSI5(Gamma,x(t,1:ncx))
	   temp=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta) - rhs
	   	   
	   Call dPsi5(Gamma,x(t,1:ncx),g)

         do j=1,n
	      fit(j)=fit(j)+temp*g(j)
	   end do
	   	      
      End Do

      if (MaxVal(dabs(fit)) .gt. (10.**MaxExp2)) FError=1
	
	Return

	End Subroutine


C Fit1b(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fit1a

      Subroutine Fit1b(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call Fit1a(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
      Return
	END SUBROUTINE



C Fit2b(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fit2a

      Subroutine Fit2b(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call Fit2a(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
      Return
	END SUBROUTINE

C Fit3b(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fit3a

      Subroutine Fit3b(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call Fit3a(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
C      Fit=0.5*Dot_Product(Fvec,Fvec)
      Return
	END SUBROUTINE


C Fit4b(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fit4a

      Subroutine Fit4b(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call Fit4a(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
      Return
	END SUBROUTINE

C Fit5b(m,Gamma,Fit)
C 
C Returns the maximal absolute elemente of Fit returned by Fit5a

      Subroutine Fit5b(m,Gamma,Fit)    

      use MNR      
	Integer(4) m
	Real(8) Gamma(m), Fit
            
	Real(8) Fvec(m)      
	
	Call Fit5a(m,m,Gamma,Fvec)

	if (FError .ne. 0) then
	  fit=-1.0E0
	  return
	endif
      Fit=MaxVal(dabs(Fvec))
      Return
	END SUBROUTINE



C DMStat

      Subroutine DMStat(l,length,dm,chi)
                       
	use Def1
	use IMSLF90

	Integer(4) l, length
	Real(8) DM(1,1), Chi(1:2), p(1:2)

	Integer(4) t, t1, j, m
	Real(8) evec(1:length,1), z(1:length,1+2*l), Amat(1:2*l+1,1:2*l+1),
     +        Bvec(1:2*l+1,1), Avec(1:2*l+1,1), Help(1:2*l+1,1), DCHIIN

      m=1+2*l
      t1=length+ta+l-1

      Do t=ta+l,t1
         
	   evec(t-ta-l+1,1)=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +                   (x(t+1,1)**(alpha-1)))- beta*lm(t+1)*(1-delta)
     +    - (u(t)**(-eta)) + lm(t)
         z(t-ta-l+1,1)=1.0
	   Do j=1,l
	      z(t-ta-l+1,1+j)=u(t-j)
	      z(t-ta-l+1,1+l+j)=x(t-j,2)
	   End Do

	End Do

      Avec=MatMul(Transpose(z),evec)
      Amat=0
	Do t=1,length
	   Help(1:m,1)=z(t,1:m)
	   Amat(1:m,1:m)=Amat(1:m,1:m)+(evec(t,1)**2)*
     +   MatMul(Help,Transpose(Help))
	End Do

	Call DLINRG(m,Amat,m,Amat,m)

      Bvec=MatMul(Amat,Avec)
	DM=MatMul(Transpose(Avec),Bvec)
     
	p(1)=chi(1)
	p(2)=chi(2)
      
	chi(1)=DCHIIN(p(1),dble(2*l+1))
	chi(2)=DCHIIN(p(2),dble(2*l+1))

	Return

	End Subroutine

      Real(8) Function GetSSE(Gamma,Psi)

	use Def1	
      use MNR
	
	Real(8) Gamma(ng)
    
      Integer(4) t
	Real(8) rhs, psi

	External Psi
      
	x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))

      FError=0

	do t=1,nrx-1

	       rhs=PSI(Gamma,x(t,1:ncx))
            if (FError .ne. 0) then
	         GetSSE=-1.0
	         return
	      endif
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

	end do

	do t=ta,nrx-2
	   rhs=beta*(u(t+1)**(-eta))*(1-delta+alpha*x(t+1,2)*
     +          (x(t+1,1)**(alpha-1))) - beta*lm(t+1)*(1-delta)
     +         - PSI(Gamma,x(t,1:ncx))
	   GetSSE=GetSSE+(rhs**2.0)

      end do
      GetSSE=dble(GetSSE/nobs)
	Return

	End Function


      Subroutine Simulate(nofs,lags,length,freq,m,Gamma,Psi)
	                         
      use Def1
	use MNR

	Integer(4) nofs, lags, freq(3), m, length
      Real(8) Gamma(m), Psi

      Integer(4) t, i, t1
	Real(8) rhs, dm, chi(2)

	External Psi

	freq=0
	t1=length+ta+lags

C Loop over number of simulations
      Do i=1, nofs,1

C Draw shocks
	   Call DRNNOR(t1+1,eps) ! Draw shocks from a standard normal
	   X(1,2)=exp(Sigma*eps(1))
         Do t=1,t1
	      X(t+1,2)=(X(t,2)**Rho)*exp(Sigma*eps(t))
	   End Do


C Generate Time Series

      x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      
	FError=0
	
      do t=1,t1

	       rhs=PSI(Gamma,x(t,1:ncx))
	       if (FError .ne. 0) goto 2
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

	end do	
      freq(3)=freq(3)+1

      chi(1)=0.05
	chi(2)=0.95

      Call DMStat(lags,length,dm,chi)

      if (dm .lt. chi(1)) freq(1)=freq(1)+1
	if (dm .gt. chi(2)) freq(2)=freq(2)+1

2     continue

      End Do

	Return

	End Subroutine
      
	
      Subroutine Tests(Gamma,Psi,t1)

	use Def1

	Integer(4) t1
	Real(8) Gamma(ng), Psi

      Integer(4) t
	Real(8) kstar, zstar, cstar, rhs, xstar(2), k2, c2, l2

	External Psi

C Compute stationary solution
	kstar=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
	zstar=1.0
	cstar=(kstar**alpha)-delta*kstar

	xstar(1)=kstar
	xstar(2)=zstar

             rhs=PSI(Gamma,xstar)
	       
	      c2=rhs**(-1.0/eta)
        k2=zstar*(kstar**alpha)+(1-delta)*kstar - c2
	       
C here comes the code that implements the Kuhn-Tucker conditions

        if ( k2 .lt. (1-delta)*kstar ) then
	          c2=zstar*(kstar**alpha)
	         l2=(c2**(-eta))-rhs
	      k2=(1-delta)*kstar
	  else
	      l2=0.0
	  endif

        write(15,"('kstar= ',F10.5,' k2=',F10.5,' cstar=',F10.5,' c2=',
     +      F10.5)") kstar,k2,cstar,c2

C Compute Impulse responses to a one-time technology shock
      open(16,file='Impulse.txt')

      X(1,2)=exp(Sigma)
         Do t=1,t1
	      X(t+1,2)=(X(t,2)**Rho)
	   End Do


C Generate Time Series

      x(1,1)=((1.0-beta*(1.0-delta))/(alpha*beta))**(1.0/(alpha-1.0))
      	
	
      do t=1,t1

	       rhs=PSI(Gamma,x(t,1:ncx))
	      
	      u(t)=rhs**(-1.0/eta)
        x(t+1,1)=x(t,2)*(x(t,1)**alpha)+(1-delta)*x(t,1) - u(t)

C here comes the code that implements the Kuhn-Tucker conditions

        if ( x(t+1,1) .lt. (1-delta)*x(t,1) ) then
	          u(t)=x(t,2)*(x(t,1)**alpha)
	         lm(t)=(u(t)**(-eta))-rhs
	      x(t+1,1)=(1-delta)*x(t,1)
	  else
	      lm(t)=0.0
	  endif

        write(16,"(3(F10.5,2X))") x(t,2), x(t,1), u(t)

	end do	
      

	Return

	End Subroutine